package tests.junit.screens;

import junit.framework.TestCase;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.rpl.ScreenRequest;
import org.objectweb.telosys.rpl.ScreenResponse;
import org.objectweb.telosys.screen.core.ScreenContext;
import org.objectweb.telosys.screen.core.ScreenData;
import org.objectweb.telosys.screen.core.ScreenDataElements;
import org.objectweb.telosys.tunit.ActionObjects;
import org.objectweb.telosys.tunit.ActionTestManager;
import org.telosys.samples.screen.author.AuthorData;
import org.telosys.samples.vo.bean.Author;

/**
 * This class is designed to test the "Author" screen behavior on the server side <br>
 * It uses the Telosys Unit framework ( "TUnit" ) to submit "ScreenRequest" <br>
 * It allows to test : <br>
 * . the Screen Manager, <br>
 * . the Screen Triggers (if any) <br>
 * . the Screen Procedures (if any) <br>
 *
 */
public class AuthorUnitTest extends TestCase 
{
	private ActionTestManager atm = null ;

	private static final String SCREEN_NAME = "author" ;
	private static final int    SCREEN_ID   = 1 ;
	
	/* 
     * Called before each test
     */
    public void setUp() 
    {    	
    	System.out.println("setUp() : test name = " + this.getName() );
    	atm = UnitTestEnv.getCurrentActionTestManager();
    	//atm = UnitTestEnv.getNewActionTestManager();
    }

    public void test() throws TelosysException
    {
    	System.out.println("test()...");
    	open(); // The screen must be open before any other request
    	load();
    	get();
    	clear();
    	set();
    	exec_action1();
    	exec_action2();
    }
    
	/* 
     * Called after each test
     */
    public void tearDown() 
    {
    	System.out.println("tearDown() : test name = " + this.getName() ) ;
    	System.out.println("---");
    }

    /**
     * Action "open"
     * @throws TelosysException
     */
    private void open() throws TelosysException
    {
    	System.out.println("open()...");
    	ScreenRequest screenRequest = atm.newScreenRequest( SCREEN_NAME, SCREEN_ID, "open" );
		
    	ActionObjects actionObjects = atm.executeAction(screenRequest);
		
    	ScreenContext screenContext = actionObjects.getScreenContext();
		System.out.println("ScreenContext : \n" + screenContext );
		
		ScreenResponse screenResponse = actionObjects.getScreenResponse();
		assertEquals( 0, screenResponse.getReturnCode() ); // Check Ret Code is OK
		assertEquals( "open", screenResponse.getAction() ); // Check response action name
    }

    /**
     * Action "load"
     * @throws TelosysException
     */
    private void load() throws TelosysException
    {
    	System.out.println("load()...");
    	ScreenRequest screenRequest = atm.newScreenRequest(SCREEN_NAME, SCREEN_ID, "load" );
		screenRequest.setParameter("id", "3");
		
		ActionObjects actionObjects = atm.executeAction(screenRequest);
		
		ScreenContext screenContext = actionObjects.getScreenContext();
		System.out.println("ScreenContext : \n" + screenContext );
		screenContext.getScreenId();
		screenContext.getScreenName();
		System.out.println("ScreenContext.getCurrentData() : \n" + screenContext.getCurrentData() );
    	
		ScreenResponse screenResponse = actionObjects.getScreenResponse();
		assertEquals( 0, screenResponse.getReturnCode() );
		assertEquals( "load", screenResponse.getAction() );
		
		ScreenData screenData = screenContext.getCurrentData() ;
		Object o = screenData.getElement(AuthorData.AUTHOR); 
		assertNotNull(o); // The element must be here after the load
		assertTrue( o instanceof Author ); // Check class
		Author author = (Author) o ;
		assertEquals( 3, author.getId() ); // Check Author ID
    }

    /**
     * Action "get"
     * @throws TelosysException
     */
    private void get() throws TelosysException
    {
    	System.out.println("get()...");
    	ScreenRequest screenRequest = atm.newScreenRequest(SCREEN_NAME, SCREEN_ID, "get" );

    	screenRequest.setElements(AuthorData.AUTHOR); 

		System.out.println("----- EXECUTE ACTION -----");
		System.out.println("-- REQUEST :\n" + screenRequest);
		ActionObjects actionObjects = atm.executeAction(screenRequest);
		
		ScreenContext screenContext = actionObjects.getScreenContext();
		System.out.println("ScreenContext : \n" + screenContext );
		screenContext.getScreenId();
		screenContext.getScreenName();
		System.out.println("ScreenContext.getCurrentData() : \n" + screenContext.getCurrentData() );
		
		ScreenResponse screenResponse = actionObjects.getScreenResponse() ;
		System.out.println("-- RESPONSE :\n" + screenResponse );
		ScreenDataElements dataElements = screenResponse.getDataElements();
		System.out.println("Response dataElements : \n" + dataElements );
		
    }

    /**
     * Action "clear"
     * @throws TelosysException
     */
    private void clear() throws TelosysException
    {
    	ScreenRequest screenRequest = atm.newScreenRequest(SCREEN_NAME, SCREEN_ID, "clear" );
		
		System.out.println("----- EXECUTE ACTION -----");
		System.out.println("-- REQUEST :\n" + screenRequest);
		ActionObjects actionObjects = atm.executeAction(screenRequest);
		ScreenContext screenContext = actionObjects.getScreenContext();
		System.out.println("ScreenContext : \n" + screenContext );
		System.out.println("ScreenContext.getCurrentData() : \n" + screenContext.getCurrentData() );
		
		ScreenResponse screenResponse = actionObjects.getScreenResponse() ;
		System.out.println("-- RESPONSE :\n" + screenResponse );
    }

    /**
     * Action "set"
     * @throws TelosysException
     */
    private void set() throws TelosysException
    {
    	ScreenRequest screenRequest = atm.newScreenRequest(SCREEN_NAME, SCREEN_ID, "set" );
		//screenRequest.setAction("set");
		Author author = new Author();
		author.setId(456);
		author.setFirstName("Hector");
		author.setLastName("Berlioz");

		screenRequest.storeDataElement(AuthorData.AUTHOR, author);

		ActionObjects actionObjects = atm.executeAction(screenRequest);
		ScreenContext screenContext = actionObjects.getScreenContext();
		System.out.println("ScreenContext : \n" + screenContext );
		screenContext.getScreenId();
		screenContext.getScreenName();
		System.out.println("ScreenContext.getCurrentData() : \n" + screenContext.getCurrentData() );
    }

    /**
     * Procedure "action1"
     * @throws TelosysException
     */
    private void exec_action1() throws TelosysException
    {
    	ScreenRequest screenRequest = atm.newScreenRequest(SCREEN_NAME, SCREEN_ID, "exec" );
		screenRequest.setProcName("action1");
		screenRequest.setParameter("p1", "A");
		screenRequest.setParameter("p2", "B");
		
		ActionObjects actionObjects = atm.executeAction(screenRequest);

		ScreenContext screenContext = actionObjects.getScreenContext();
		System.out.println("ScreenContext : \n" + screenContext );
		screenContext.getScreenId();
		screenContext.getScreenName();
		
		ScreenResponse screenResponse = actionObjects.getScreenResponse();
		System.out.println( "Ret code = " + screenResponse.getReturnCode() );
		System.out.println( "Ret msg  = " + screenResponse.getReturnMessage() );
		
		assertEquals( 0, screenResponse.getReturnCode() );

    }

    /**
     * Procedure "action2"
     * @throws TelosysException
     */
    private void exec_action2() throws TelosysException
    {
    	ScreenRequest screenRequest = atm.newScreenRequest(SCREEN_NAME, SCREEN_ID, "exec" );
		screenRequest.setProcName("action2");
		
		ActionObjects actionObjects = atm.executeAction(screenRequest);

		ScreenResponse screenResponse = actionObjects.getScreenResponse();
		System.out.println( "Ret code = " + screenResponse.getReturnCode() );
		System.out.println( "Ret msg  = " + screenResponse.getReturnMessage() );

		assertEquals( 0, screenResponse.getReturnCode() ); // ERROR ( no "action2" procedure )

    }
}
